@@ -63,6 +63,7 @@
 #define KEY_INTERVAL       "repeat-interval"
 #define KEY_DELAY          "delay"
 #define KEY_CLICK_VOLUME   "click-volume"
+#define KEY_REMEMBER_NUMLOCK_STATE "remember-numlock-state"
 #define KEY_NUMLOCK_STATE  "numlock-state"
 
 #define KEY_BELL_VOLUME    "bell-volume"
@@ -115,6 +116,8 @@ static gboolean apply_input_sources_settings     (GSettings               *setti
                                                   gpointer                 keys,
                                                   gint                     n_keys,
                                                   GsdKeyboardManager      *manager);
+static void     set_gtk_im_module                (GsdKeyboardManager      *manager,
+                                                  const gchar             *new_module);
 
 G_DEFINE_TYPE (GsdKeyboardManager, gsd_keyboard_manager, G_TYPE_OBJECT)
 
@@ -179,14 +182,16 @@ fetch_ibus_engines_result (GObject            *object,
 {
         GsdKeyboardManagerPrivate *priv = manager->priv;
         GList *list, *l;
-        GError *error;
+        GError *error = NULL;
+
+        /* engines shouldn't be there yet */
+        g_return_if_fail (priv->ibus_engines == NULL);
+
+        g_clear_object (&priv->ibus_cancellable);
 
-        error = NULL;
         list = ibus_bus_list_engines_async_finish (priv->ibus,
                                                    result,
                                                    &error);
-        g_clear_object (&priv->ibus_cancellable);
-
         if (!list && error) {
                 g_warning ("Couldn't finish IBus request: %s", error->message);
                 g_error_free (error);
@@ -224,6 +229,10 @@ fetch_ibus_engines (GsdKeyboardManager *manager)
 {
         GsdKeyboardManagerPrivate *priv = manager->priv;
 
+        /* engines shouldn't be there yet */
+        g_return_if_fail (priv->ibus_engines == NULL);
+        g_return_if_fail (priv->ibus_cancellable == NULL);
+
         priv->ibus_cancellable = g_cancellable_new ();
 
         ibus_bus_list_engines_async (priv->ibus,
@@ -285,6 +294,9 @@ got_session_name (GObject            *object,
         const gchar *session_name = NULL;
         GError *error = NULL;
 
+        /* IBus shouldn't have been touched yet */
+        g_return_if_fail (priv->ibus == NULL);
+
         g_clear_object (&priv->ibus_cancellable);
 
         result = g_dbus_connection_call_finish (connection, res, &error);
@@ -317,6 +329,9 @@ got_bus (GObject            *object,
         GsdKeyboardManagerPrivate *priv = manager->priv;
         GError *error = NULL;
 
+        /* IBus shouldn't have been touched yet */
+        g_return_if_fail (priv->ibus == NULL);
+
         g_clear_object (&priv->ibus_cancellable);
 
         connection = g_bus_get_finish (res, &error);
@@ -345,6 +360,46 @@ got_bus (GObject            *object,
                                 manager);
 }
 
+static void
+set_ibus_engine_finish (GObject            *object,
+                        GAsyncResult       *res,
+                        GsdKeyboardManager *manager)
+{
+        gboolean result;
+        IBusBus *ibus = IBUS_BUS (object);
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+        GError *error = NULL;
+
+        g_clear_object (&priv->ibus_cancellable);
+
+        result = ibus_bus_set_global_engine_async_finish (ibus, res, &error);
+        if (!result) {
+                g_warning ("Couldn't set IBus engine: %s", error->message);
+                g_error_free (error);
+        }
+}
+
+static void
+set_ibus_engine (GsdKeyboardManager *manager,
+                 const gchar        *engine_id)
+{
+        GsdKeyboardManagerPrivate *priv = manager->priv;
+
+        g_return_if_fail (priv->ibus != NULL);
+        g_return_if_fail (priv->ibus_engines != NULL);
+
+        g_cancellable_cancel (priv->ibus_cancellable);
+        g_clear_object (&priv->ibus_cancellable);
+        priv->ibus_cancellable = g_cancellable_new ();
+
+        ibus_bus_set_global_engine_async (priv->ibus,
+                                          engine_id,
+                                          -1,
+                                          priv->ibus_cancellable,
+                                          (GAsyncReadyCallback)set_ibus_engine_finish,
+                                          manager);
+}
+
 static void
 set_ibus_xkb_engine (GsdKeyboardManager *manager,
                      const gchar        *xkb_id)
@@ -359,7 +414,7 @@ set_ibus_xkb_engine (GsdKeyboardManager *manager,
         if (!engine)
                 return;
 
-        ibus_bus_set_global_engine (priv->ibus, ibus_engine_desc_get_name (engine));
+        set_ibus_engine (manager, ibus_engine_desc_get_name (engine));
 }
 #endif  /* HAVE_IBUS */
 
@@ -419,6 +474,21 @@ numlock_set_xkb_state (GsdNumLockState new_state)
         XkbLockModifiers (dpy, XkbUseCoreKbd, num_mask, new_state == GSD_NUM_LOCK_STATE_ON ? num_mask : 0);
 }
 
+static const char *
+num_lock_state_to_string (GsdNumLockState numlock_state)
+{
+	switch (numlock_state) {
+	case GSD_NUM_LOCK_STATE_UNKNOWN:
+		return "GSD_NUM_LOCK_STATE_UNKNOWN";
+	case GSD_NUM_LOCK_STATE_ON:
+		return "GSD_NUM_LOCK_STATE_ON";
+	case GSD_NUM_LOCK_STATE_OFF:
+		return "GSD_NUM_LOCK_STATE_OFF";
+	default:
+		return "UNKNOWN";
+	}
+}
+
 static GdkFilterReturn
 xkb_events_filter (GdkXEvent *xev_,
 		   GdkEvent  *gdkev_,
@@ -440,6 +510,9 @@ xkb_events_filter (GdkXEvent *xev_,
 		numlock_state = (num_mask & locked_mods) ? GSD_NUM_LOCK_STATE_ON : GSD_NUM_LOCK_STATE_OFF;
 
 		if (numlock_state != manager->priv->old_state) {
+			g_debug ("New num-lock state '%s' != Old num-lock state '%s'",
+				 num_lock_state_to_string (numlock_state),
+				 num_lock_state_to_string (manager->priv->old_state));
 			g_settings_set_enum (manager->priv->settings,
 					     KEY_NUMLOCK_STATE,
 					     numlock_state);
@@ -590,6 +663,9 @@ replace_layout_and_variant (GsdKeyboardManager *manager,
         const gchar *locale;
         gchar *language;
 
+        if (!layout)
+                return;
+
         locale = setlocale (LC_MESSAGES, NULL);
         /* If LANG is empty, default to en_US */
         if (!locale)
@@ -685,9 +761,9 @@ add_xkb_options (GsdKeyboardManager *manager,
 }
 
 static void
-apply_xkb_layout (GsdKeyboardManager *manager,
-                  const gchar        *layout,
-                  const gchar        *variant)
+apply_xkb_settings (GsdKeyboardManager *manager,
+                    const gchar        *layout,
+                    const gchar        *variant)
 {
         XkbRF_RulesRec *xkb_rules;
         XkbRF_VarDefsRec *xkb_var_defs;
@@ -746,6 +822,7 @@ apply_input_sources_settings (GSettings          *settings,
         GsdKeyboardManagerPrivate *priv = manager->priv;
         GVariant *sources;
         guint current;
+        guint n_sources;
         const gchar *type = NULL;
         const gchar *id = NULL;
         const gchar *layout = NULL;
@@ -753,16 +830,15 @@ apply_input_sources_settings (GSettings          *settings,
 
         sources = g_settings_get_value (priv->input_sources_settings, KEY_INPUT_SOURCES);
         current = g_settings_get_uint (priv->input_sources_settings, KEY_CURRENT_INPUT_SOURCE);
+        n_sources = g_variant_n_children (sources);
 
-        if (g_variant_n_children (sources) < 1) {
-                g_warning ("No input source configured, resetting");
-                g_settings_reset (priv->input_sources_settings, KEY_INPUT_SOURCES);
+        if (n_sources < 1)
                 goto exit;
-        }
-        if (current >= g_variant_n_children (sources)) {
+
+        if (current >= n_sources) {
                 g_settings_set_uint (priv->input_sources_settings,
                                      KEY_CURRENT_INPUT_SOURCE,
-                                     g_variant_n_children (sources) - 1);
+                                     n_sources - 1);
                 goto exit;
         }
 
@@ -803,22 +879,17 @@ apply_input_sources_settings (GSettings          *settings,
                         goto exit;
                 }
 
-                if (ibus_bus_set_global_engine (priv->ibus, id))
-                        set_gtk_im_module (manager, GTK_IM_MODULE_IBUS);
-                else
-                        g_warning ("Couldn't set IBus engine '%s'", id);
+                set_gtk_im_module (manager, GTK_IM_MODULE_IBUS);
+                set_ibus_engine (manager, id);
 #else
                 g_warning ("IBus input source type specified but IBus support was not compiled");
-                goto exit;
 #endif
         } else {
                 g_warning ("Unknown input source type '%s'", type);
-                goto exit;
         }
 
-        apply_xkb_layout (manager, layout, variant);
-
  exit:
+        apply_xkb_settings (manager, layout, variant);
         g_variant_unref (sources);
         /* Prevent individual "changed" signal invocations since we
            don't need them. */
@@ -826,34 +897,83 @@ apply_input_sources_settings (GSettings          *settings,
 }
 
 static void
-apply_settings (GSettings          *settings,
-                const char         *key,
-                GsdKeyboardManager *manager)
+apply_bell (GsdKeyboardManager *manager)
 {
+	GSettings       *settings;
         XKeyboardControl kbdcontrol;
-        gboolean         repeat;
         gboolean         click;
-        guint            interval;
-        guint            delay;
-        int              click_volume;
         int              bell_volume;
         int              bell_pitch;
         int              bell_duration;
         GsdBellMode      bell_mode;
-        gboolean         rnumlock;
+        int              click_volume;
 
-        repeat        = g_settings_get_boolean  (settings, KEY_REPEAT);
+        g_debug ("Applying the bell settings");
+        settings      = manager->priv->settings;
         click         = g_settings_get_boolean  (settings, KEY_CLICK);
-        interval      = g_settings_get_uint  (settings, KEY_INTERVAL);
-        delay         = g_settings_get_uint  (settings, KEY_DELAY);
         click_volume  = g_settings_get_int   (settings, KEY_CLICK_VOLUME);
+
         bell_pitch    = g_settings_get_int   (settings, KEY_BELL_PITCH);
         bell_duration = g_settings_get_int   (settings, KEY_BELL_DURATION);
 
         bell_mode = g_settings_get_enum (settings, KEY_BELL_MODE);
         bell_volume   = (bell_mode == GSD_BELL_MODE_ON) ? 50 : 0;
 
-        rnumlock      = g_settings_get_boolean  (settings, "remember-numlock-state");
+        /* as percentage from 0..100 inclusive */
+        if (click_volume < 0) {
+                click_volume = 0;
+        } else if (click_volume > 100) {
+                click_volume = 100;
+        }
+        kbdcontrol.key_click_percent = click ? click_volume : 0;
+        kbdcontrol.bell_percent = bell_volume;
+        kbdcontrol.bell_pitch = bell_pitch;
+        kbdcontrol.bell_duration = bell_duration;
+
+        gdk_error_trap_push ();
+        XChangeKeyboardControl (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                KBKeyClickPercent | KBBellPercent | KBBellPitch | KBBellDuration,
+                                &kbdcontrol);
+
+        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
+        gdk_error_trap_pop_ignored ();
+}
+
+static void
+apply_numlock (GsdKeyboardManager *manager)
+{
+	GSettings *settings;
+        gboolean rnumlock;
+
+        g_debug ("Applying the num-lock settings");
+        settings = manager->priv->settings;
+        rnumlock = g_settings_get_boolean  (settings, KEY_REMEMBER_NUMLOCK_STATE);
+        manager->priv->old_state = g_settings_get_enum (manager->priv->settings, KEY_NUMLOCK_STATE);
+
+        gdk_error_trap_push ();
+        if (rnumlock) {
+                g_debug ("Remember num-lock is set, so applying setting '%s'",
+                         num_lock_state_to_string (manager->priv->old_state));
+                numlock_set_xkb_state (manager->priv->old_state);
+        }
+
+        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
+        gdk_error_trap_pop_ignored ();
+}
+
+static void
+apply_repeat (GsdKeyboardManager *manager)
+{
+	GSettings       *settings;
+        gboolean         repeat;
+        guint            interval;
+        guint            delay;
+
+        g_debug ("Applying the repeat settings");
+        settings      = manager->priv->settings;
+        repeat        = g_settings_get_boolean  (settings, KEY_REPEAT);
+        interval      = g_settings_get_uint  (settings, KEY_INTERVAL);
+        delay         = g_settings_get_uint  (settings, KEY_DELAY);
 
         gdk_error_trap_push ();
         if (repeat) {
@@ -870,29 +990,45 @@ apply_settings (GSettings          *settings,
                 XAutoRepeatOff (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
         }
 
-        /* as percentage from 0..100 inclusive */
-        if (click_volume < 0) {
-                click_volume = 0;
-        } else if (click_volume > 100) {
-                click_volume = 100;
-        }
-        kbdcontrol.key_click_percent = click ? click_volume : 0;
-        kbdcontrol.bell_percent = bell_volume;
-        kbdcontrol.bell_pitch = bell_pitch;
-        kbdcontrol.bell_duration = bell_duration;
-        XChangeKeyboardControl (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                KBKeyClickPercent | KBBellPercent | KBBellPitch | KBBellDuration,
-                                &kbdcontrol);
-
-        manager->priv->old_state = g_settings_get_enum (manager->priv->settings, KEY_NUMLOCK_STATE);
-
-        if (rnumlock)
-                numlock_set_xkb_state (manager->priv->old_state);
-
         XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
         gdk_error_trap_pop_ignored ();
 }
 
+static void
+apply_all_settings (GsdKeyboardManager *manager)
+{
+	apply_repeat (manager);
+	apply_bell (manager);
+	apply_numlock (manager);
+}
+
+static void
+settings_changed (GSettings          *settings,
+                  const char         *key,
+                  GsdKeyboardManager *manager)
+{
+	if (g_strcmp0 (key, KEY_CLICK) == 0||
+	    g_strcmp0 (key, KEY_CLICK_VOLUME) == 0 ||
+	    g_strcmp0 (key, KEY_BELL_PITCH) == 0 ||
+	    g_strcmp0 (key, KEY_BELL_DURATION) == 0 ||
+	    g_strcmp0 (key, KEY_BELL_MODE) == 0) {
+		g_debug ("Bell setting '%s' changed, applying bell settings", key);
+		apply_bell (manager);
+	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0 ||
+		 g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
+		g_debug ("Num-Lock state '%s' changed, applying num-lock state settings", key);
+		apply_numlock (manager);
+	} else if (g_strcmp0 (key, KEY_REPEAT) == 0 ||
+		 g_strcmp0 (key, KEY_INTERVAL) == 0 ||
+		 g_strcmp0 (key, KEY_DELAY) == 0) {
+		g_debug ("Key repeat setting '%s' changed, applying key repeat settings", key);
+		apply_repeat (manager);
+	} else {
+		g_warning ("Unhandled settings change, key '%s'", key);
+	}
+
+}
+
 static void
 device_added_cb (GdkDeviceManager   *device_manager,
                  GdkDevice          *device,
@@ -902,7 +1038,8 @@ device_added_cb (GdkDeviceManager   *device_manager,
 
         source = gdk_device_get_source (device);
         if (source == GDK_SOURCE_KEYBOARD) {
-                apply_settings (manager->priv->settings, NULL, manager);
+                g_debug ("New keyboard plugged in, applying all settings");
+                apply_all_settings (manager);
                 apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
                 run_custom_command (device, COMMAND_DEVICE_ADDED);
         }
@@ -965,10 +1102,11 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
         apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
 #endif
         /* apply current settings before we install the callback */
-        apply_settings (manager->priv->settings, NULL, manager);
+        g_debug ("Started the keyboard plugin, applying all settings");
+        apply_all_settings (manager);
 
         g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
-                          G_CALLBACK (apply_settings), manager);
+                          G_CALLBACK (settings_changed), manager);
         g_signal_connect (G_OBJECT (manager->priv->input_sources_settings), "change-event",
                           G_CALLBACK (apply_input_sources_settings), manager);
 
